<?php
/*
Plugin Name: RegLevel
Plugin URI: http://www.jumping-duck.com/wordpress/
Description: Allows you to specify different registration screens for each user level.  Please read about changes between version 0.3 and version 1.0 before upgrading!  The way you specify user levels and registration screens has completely changed!  To set up RegLevel redirects, visit <a href="users.php?page=reglevel/includes/admin.php">Users &raquo; RegLevel</a>.
Version: 1.1.1
Author: Eric Mann
Author URI: http://www.eamann.com
License: GPLv2+
*/

/*  Copyright 2008-9  ERIC MANN  (email : eric@eamann.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or 
	(at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*  New in version 1.1.1

	Fixes error on admin pages that was preventing redirects from being saved to the
	database.  Special thanks to Keith Bieneman at www.tilehaus.com for reporting this
	bug!
 */

/* 	New features in version 1.1

	THIS VERSION DISCONTINUES SUPPORT OF THE LEGACY SHORTCODE OPTION!!!  PLEASE UPDATE
	YOUR SITE TO USE THE MORE ACCURATE, SAFER REDIRECT METHOD!!!

	The redirect management page now allows for deletion of redirects.  You can	also change 
	some WordPress default options (i.e. enable/disable registration, change default user 
	type) from the RegLevel option screen.
	
	We've also updated the plug-in to work with Register Plus.
 */

/*  New features in version 1.0

	This is a major upgrade!
	
	Previous versions of RegLevel have used shortcodes to create registration redirects.
	This is insecure, bulky, and causes significant problems on slower or heavier-traffic
	servers.  The NEW version creates a cookie that follow the user when they try to register.  
	This is a safer, lighter, and easier way to set things up.
	
	Also, you no longer need to create a new page manually to create a redirect.  Instead,
	use the built-in administration screens to set the URLs you will use as a redirect.
	
	For example, you can set http://www.bloglink.com/?reglevel=023adE as a redirect for the
	"Author" user type.  It will function without any additional effort on your part!
	
	The plugin is fully backwards compatable, so if you still have redirect pages on your
	system there is no need to remove or recode them.  Just make sure you use the new method
	in the future!
 */

/*  New features in version 0.3

	This is actually a bug fix, correcting the "Headers already sent" error many people
	have been experiencing since WordPress version 2.6.3.  There are no significant new
	features.
 */

/*  New features in version 0.2

	Rather than limit users to one secondary registration screen, we are adding
	functionality to the [reglevel] shortcode.  Now, you can specify the user level
	directly in the shortcode on the page.  This gives you the ability to have as many
	secondary registration pages as you want!
	
	If no user level is set in the shortcode, the user level reverts to the default user
	level set on the RegLevel administration page.
 */
require_once('includes/functions.php');
function reg_install() {
	/**
	 * The first step is to update the plugin if someone installed a previous version.
	 * We do this by checking old variables left over from versions 0.1 and 0.2 and deleting them
	 * if they exist.
	 */
	$rl_page = get_option('plugin_reglevel_page');		//Used in version 0.1
	$rl_user = get_option('plugin_reglevel_user');		//Used in version 0.1
	$rl_def = get_option('plugin_reglevel_default');	//Used in version 0.2
	$rl_set = get_option('plugin_reglevel_set');		//Used in version 0.2
	$rl_ver = get_option('plugin_reglevel_version');	//Used in version 0.2
	if ( $rl_page || $rl_user || $rl_def || $rl_set || $rl_ver ) {
		delete_option('plugin_reglevel_user');
		delete_option('plugin_reglevel_page');
		delete_option('plugin_reglevel_default');
		delete_option('plugin_reglevel_set');
		delete_option('plugin_reglevel_version');
	}

	/**
	 * Now we check to see if version 1.0 has already been installed.  If yes, we do nothing.
	 * If no, then we install the system.
	 */
	$reg_version = get_option('RegLevel_Version');
	if ( ! $reg_version ) {
		add_option('RegLevel_Version', '1.0');
		$reg_new_install=true;
	}
	
	/** 
	 * Next we have two separate things to check.  First, if the user is upgrading from a previous
	 * version, we need to record this in the database and call the legacy shortcode functions.  These
	 * will be called always to make sure we retain backwards compatability.  Second, we need to run
	 * some table installation scripts if this is a new installation.  This will set up our references
	 * for the redirect pages.
	 */
	if( $reg_new_install==true )
		include('includes/install.php');	//This runs the database installation scripts.  We only need it once
}
register_activation_hook(__FILE__,'reg_install');

/**
 * The following code creates our admin menu and lets users specify options
 *
 */
add_action('admin_menu', 'rl_add_pages');
function rl_add_pages() {
    add_submenu_page('users.php','RegLevel Settings', 'RegLevel', 8, 'reglevel/includes/admin.php');
}

/**  
  *  The following sets of functions are the 'new' core to the RegLevel plugin.
  */

define('REGLEVEL_REWRITERULES', '1');								// flag to determine if plugin can change WP rewrite rules
define('REGLEVEL_QUERYVAR', 'reglevel');							// get/post variable name for querying tag/keyword from WP
define('REGLEVEL_META', 'regpage');									// post meta key used in the wp database
define('REGLEVEL_TEMPLATE', '');									// template file to use for registration pages - should be NULL

function reglevel_init() {
    global $wp_rewrite;
    
    /* Shouldn't need to change this - can set to 0 if you want to force permalinks off */
    if (isset($wp_rewrite) && $wp_rewrite->using_permalinks()) {
        define('REGLEVEL_REWRITEON', '1');							// turn on nice permalinks
        define('REGLEVEL_LINKBASE', $wp_rewrite->root);				// set to "index.php/" if using that style
    } else {
        define('REGLEVEL_REWRITEON', '0');							// old school links
        define('REGLEVEL_LINKBASE', '');							// don't need this
    }

    /* generate rewrite rules for above queries */
    if (REGLEVEL_REWRITEON && REGLEVEL_REWRITERULES)
        add_filter('search_rewrite_rules', 'reglevel_createRewriteRules');
}
add_action('init','reglevel_init');

function reglevel_createRewriteRules($rewrite) {
	global $wp_rewrite;
	
	// add rewrite tokens
	$keytag_token = '%' . REGLEVEL_QUERYVAR . '%';
	$wp_rewrite->add_rewrite_tag($keytag_token, '(.+)', REGLEVEL_QUERYVAR . '=');
    
	$reglevel_structure = $wp_rewrite->root . REGLEVEL_QUERYVAR . "/$keytag_token";
	$reglevel_rewrite = $wp_rewrite->generate_rewrite_rules($reglevel_structure);
	
	return ( $rewrite + $reglevel_rewrite );
}

function is_regpage() {
    global $wp_version;
    $regpage = ( isset($wp_version) && ($wp_version >= 2.0) ) ? 
                get_query_var(REGLEVEL_QUERYVAR) : 
                $GLOBALS[REGLEVEL_QUERYVAR];
	if (!is_null($regpage) && ($regpage != ''))
		return true;
	else
		return false;
}

add_filter('query_vars', 'reglevel_addQueryVar');
add_action('parse_query', 'reglevel_parseQuery');

function reglevel_addQueryVar($wpvar_array) {
	$wpvar_array[] = REGLEVEL_QUERYVAR;
	return($wpvar_array);
}

function reglevel_parseQuery() {
	// if this is a keyword query, then reset other is_x flags and add query filters
	if (is_regpage()) {
		global $wp_query;
		$wp_query->is_single = false;
		$wp_query->is_page = false;
		$wp_query->is_archive = false;
		$wp_query->is_search = false;
		$wp_query->is_home = false;
		
//		add_filter('posts_where', 'reglevel_postsWhere');
//		add_filter('posts_join', 'reglevel_postsJoin');
		add_action('template_redirect', 'reglevel_redirect');
	}
}

function reglevel_postsWhere($where) {
    global $wp_version;
    $regpage = ( isset($wp_version) && ($wp_version >= 2.0) ) ? 
                get_query_var(REGLEVEL_QUERYVAR) : 
                $GLOBALS[REGLEVEL_QUERYVAR];

    $where .= " AND wp_meta.meta_key = '" . REGLEVEL_META . "' ";
	$where .= " AND wp_meta.meta_value LIKE '%" . $regpage . "%' ";

    $where = str_replace(' AND (post_status = "publish"', ' AND ((post_status = \'static\' OR post_status = \'publish\')', $where);
    
	return ($where);
}

function reglevel_postsJoin($join) {
	global $wpdb;
	$join .= " LEFT JOIN $wpdb->postmeta AS jreglevel_meta ON ($wpdb->posts.ID = jreglevel_meta.post_id) ";
	return ($join);
}

function reglevel_redirect() {
	if (is_regpage()) {
		reglevel_start_session();		
		require('./wp-load.php');
		wp_redirect('wp-login.php?action=register');
	}
	return;
}

function reglevel_start_session() {
	global $wpdb;
	$regpage = get_query_var(REGLEVEL_QUERYVAR);
	$rlquery = "SELECT regleveldef FROM " . $wpdb->prefix . "reglevel WHERE pagedef = '" . $regpage . "'";
	$rlevel = $wpdb->get_var($rlquery);
	setcookie('reglevel',$rlevel,time()+600);	
}

function reglevel_promote($user_ID) {
	if(isset($_COOKIE['reglevel'])) {
		$reglevel_level = $_COOKIE['reglevel'];
		$update = 'promote';
		$role=$reglevel_level;
		$user = new WP_User($user_ID);
		$user->set_role($reglevel_level);
	}
	setcookie('reglevel','',time()-600);
	return $user_ID;	
}

add_action ( 'user_register', 'reglevel_promote' );
?>
